### Random Promotions script, by Baldyr

from CvPythonExtensions import *

### constants

gc = CyGlobalContext()
Game = CyGame()
Interface = CyInterface()
unitTypePromotionDict = dict()
promotionHintsDict = dict()
iLeaderIndex = -1

# custom settings

leaderTag = "PROMOTION_BGP"
combatMessage = "%s received the %s Promotion!"

### functions

def setup():
    global eLeader, unitCombatPromotionDict
    eLeader = gc.getInfoTypeForString(leaderTag)
    unitCombatPromotionDict = {iLeaderIndex:[]}
    for ePromotion in xrange(gc.getNumPromotionInfos()):
        pPromotionInfo = gc.getPromotionInfo(ePromotion)
        if pPromotionInfo.getPrereqPromotion() == eLeader:
            unitCombatPromotionDict[iLeaderIndex].append(ePromotion)
        else:
            for eUnitCombatType in xrange(gc.getNumUnitCombatInfos()):
                if not pPromotionInfo.isLeader() and pPromotionInfo.getUnitCombat(eUnitCombatType):
                    unitCombatPromotionDict.setdefault(eUnitCombatType, list()).append(ePromotion)

def execute(pUnit, unitInfo):
    eTeam = pUnit.getTeam()
    eUnitType = unitInfo.getID()
    eUnitCombatType = unitInfo.getUnitCombatType()
    lUnitCombatPromotions = unitCombatPromotionDict.get(eUnitCombatType, list())
    global unitTypePromotionDict, promotionHintsDict
    unitTypePromotions = unitTypePromotionDict.setdefault(eUnitType, set())
    teamPromotionHintsDict = promotionHintsDict.setdefault(eTeam, dict())
    promotionHints = teamPromotionHintsDict.setdefault(eUnitType, set())
    bUnitTypes = len(unitTypePromotions) > 0
    if not bUnitTypes or not promotionHints.issuperset(unitTypePromotions):
        if bUnitTypes:
            availablePromotions = unitTypePromotions
        else:
            availablePromotions = set(lUnitCombatPromotions)
        for ePromotion in availablePromotions.difference(promotionHints):
            if pUnit.canAcquirePromotion(ePromotion):
                promotionHints.add(ePromotion)
            if not bUnitTypes and pUnit.isPromotionValid(ePromotion):
                unitTypePromotions.add(ePromotion)
        unitTypePromotionDict[eUnitType] = unitTypePromotions
        teamPromotionHintsDict[eUnitType] = promotionHints
    lValidPromotions = leader(pUnit, list(promotionHints))
    promote(pUnit, lValidPromotions)
    
def leader(pUnit, lValidPromotions):
    if pUnit.isHasPromotion(eLeader):
        for ePromotion in unitCombatPromotionDict[iLeaderIndex]:
            if pUnit.canAcquirePromotion(ePromotion):
                lValidPromotions.append(ePromotion)
    return lValidPromotions
    
def promote(pUnit, lPromotions):
    while lPromotions:
        iRandNum = Game.getSorenRandNum(len(lPromotions), "random promotion")
        ePromotion = lPromotions[iRandNum]
        if pUnit.canAcquirePromotion(ePromotion):            
            pUnit.setHasPromotion(ePromotion, True)
            display(pUnit, ePromotion)
            return
        else:
            lPromotions.remove(ePromotion)

def display(pUnit, ePromotion):
    if gc.getPlayer(pUnit.getOwner()).isHuman():
        unitName = pUnit.getName()
        promotionName = gc.getPromotionInfo(ePromotion).getDescription()
        message = combatMessage % (unitName, promotionName)
        Interface.addImmediateMessage(message, "")
